@@ -1,23 +1,22 @@
 /* Implementation of pattern-matching file search paths for GNU Make.
 Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
-1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 Free Software
-Foundation, Inc.
+1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
+2010 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
-Foundation; either version 2, or (at your option) any later version.
+Foundation; either version 3 of the License, or (at your option) any later
+version.
 
 GNU Make is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-GNU Make; see the file COPYING.  If not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.  */
+this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "make.h"
-#include "dir_fns.h"
 #include "filedef.h"
 #include "variable.h"
 #ifdef WINDOWS32
@@ -30,10 +29,10 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.  */
 struct vpath
   {
     struct vpath *next;	/* Pointer to next struct in the linked list.  */
-    char *pattern;	/* The pattern to match.  */
-    char *percent;	/* Pointer into `pattern' where the `%' is.  */
+    const char *pattern;/* The pattern to match.  */
+    const char *percent;/* Pointer into `pattern' where the `%' is.  */
     unsigned int patlen;/* Length of the pattern.  */
-    char **searchpath;	/* Null-terminated list of directories.  */
+    const char **searchpath; /* Null-terminated list of directories.  */
     unsigned int maxlen;/* Maximum length of any entry in the list.  */
   };
 
@@ -49,11 +48,10 @@ static struct vpath *general_vpath;
 
 static struct vpath *gpaths;
 
-static int selective_vpath_search PARAMS ((struct vpath *path, char **file, FILE_TIMESTAMP *mtime_ptr));
 
-/* Reverse the chain of selective VPATH lists so they
-   will be searched in the order given in the makefiles
-   and construct the list from the VPATH variable.  */
+/* Reverse the chain of selective VPATH lists so they will be searched in the
+   order given in the makefiles and construct the list from the VPATH
+   variable.  */
 
 void
 build_vpath_lists ()
@@ -90,13 +88,14 @@ build_vpath_lists ()
     {
       /* Save the list of vpaths.  */
       struct vpath *save_vpaths = vpaths;
+      char gp[] = "%";
 
       /* Empty `vpaths' so the new one will have no next, and `vpaths'
 	 will still be nil if P contains no existing directories.  */
       vpaths = 0;
 
       /* Parse P.  */
-      construct_vpath_list ("%", p);
+      construct_vpath_list (gp, p);
 
       /* Store the created path as the general path,
 	 and restore the old list of vpaths.  */
@@ -122,13 +121,14 @@ build_vpath_lists ()
     {
       /* Save the list of vpaths.  */
       struct vpath *save_vpaths = vpaths;
+      char gp[] = "%";
 
       /* Empty `vpaths' so the new one will have no next, and `vpaths'
 	 will still be nil if P contains no existing directories.  */
       vpaths = 0;
 
       /* Parse P.  */
-      construct_vpath_list ("%", p);
+      construct_vpath_list (gp, p);
 
       /* Store the created path as the GPATH,
 	 and restore the old list of vpaths.  */
@@ -137,7 +137,7 @@ build_vpath_lists ()
     }
 }
 
-/* Construct the VPATH listing for the pattern and searchpath given.
+/* Construct the VPATH listing for the PATTERN and DIRPATH given.
 
    This function is called to generate selective VPATH lists and also for
    the general VPATH list (which is in fact just a selective VPATH that
@@ -145,9 +145,9 @@ build_vpath_lists ()
    linked list of all selective VPATH lists or in the GENERAL_VPATH
    variable.
 
-   If SEARCHPATH is nil, remove all previous listings with the same
+   If DIRPATH is nil, remove all previous listings with the same
    pattern.  If PATTERN is nil, remove all VPATH listings.  Existing
-   and readable directories that are not "." given in the searchpath
+   and readable directories that are not "." given in the DIRPATH
    separated by the path element separator (defined in make.h) are
    loaded into the directory hash table if they are not there already
    and put in the VPATH searchpath for the given pattern with trailing
@@ -159,23 +159,20 @@ build_vpath_lists ()
 void
 construct_vpath_list (char *pattern, char *dirpath)
 {
-  register unsigned int elem;
-  register char *p;
-  register char **vpath;
-  register unsigned int maxvpath;
+  unsigned int elem;
+  char *p;
+  const char **vpath;
+  unsigned int maxvpath;
   unsigned int maxelem;
-  char *percent = NULL;
+  const char *percent = NULL;
 
   if (pattern != 0)
-    {
-      pattern = xstrdup (pattern);
-      percent = find_percent (pattern);
-    }
+    percent = find_percent (pattern);
 
   if (dirpath == 0)
     {
       /* Remove matching listings.  */
-      register struct vpath *path, *lastpath;
+      struct vpath *path, *lastpath;
 
       lastpath = 0;
       path = vpaths;
@@ -195,9 +192,9 @@ construct_vpath_list (char *pattern, char *dirpath)
 		lastpath->next = next;
 
 	      /* Free its unused storage.  */
-	      free (path->pattern);
-	      free ((char *) path->searchpath);
-	      free ((char *) path);
+              /* MSVC erroneously warns without a cast here.  */
+	      free ((void *)path->searchpath);
+	      free (path);
 	    }
 	  else
 	    lastpath = path;
@@ -205,8 +202,6 @@ construct_vpath_list (char *pattern, char *dirpath)
 	  path = next;
 	}
 
-      if (pattern != 0)
-	free (pattern);
       return;
     }
 
@@ -214,6 +209,10 @@ construct_vpath_list (char *pattern, char *dirpath)
     convert_vpath_to_windows32(dirpath, ';');
 #endif
 
+  /* Skip over any initial separators and blanks.  */
+  while (*dirpath == PATH_SEPARATOR_CHAR || isblank ((unsigned char)*dirpath))
+    ++dirpath;
+
   /* Figure out the maximum number of VPATH entries and put it in
      MAXELEM.  We start with 2, one before the first separator and one
      nil (the list terminator) and increment our estimated number for
@@ -224,15 +223,11 @@ construct_vpath_list (char *pattern, char *dirpath)
     if (*p++ == PATH_SEPARATOR_CHAR || isblank ((unsigned char)*p))
       ++maxelem;
 
-  vpath = (char **) xmalloc (maxelem * sizeof (char *));
+  vpath = xmalloc (maxelem * sizeof (const char *));
   maxvpath = 0;
 
-  /* Skip over any initial separators and blanks.  */
-  p = dirpath;
-  while (*p == PATH_SEPARATOR_CHAR || isblank ((unsigned char)*p))
-    ++p;
-
   elem = 0;
+  p = dirpath;
   while (*p != '\0')
     {
       char *v;
@@ -240,37 +235,36 @@ construct_vpath_list (char *pattern, char *dirpath)
 
       /* Find the end of this entry.  */
       v = p;
-      while (*p != '\0' && *p != PATH_SEPARATOR_CHAR
+      while (*p != '\0'
+#if defined(HAVE_DOS_PATHS) && (PATH_SEPARATOR_CHAR == ':')
+	     /* Platforms whose PATH_SEPARATOR_CHAR is ':' and which
+		also define HAVE_DOS_PATHS would like us to recognize
+		colons after the drive letter in the likes of
+		"D:/foo/bar:C:/xyzzy".  */
+	     && (*p != PATH_SEPARATOR_CHAR
+		 || (p == v + 1 && (p[1] == '/' || p[1] == '\\')))
+#else
+	     && *p != PATH_SEPARATOR_CHAR
+#endif
 	     && !isblank ((unsigned char)*p))
 	++p;
 
       len = p - v;
       /* Make sure there's no trailing slash,
 	 but still allow "/" as a directory.  */
-#if defined(__MSDOS__) || defined(__EMX__)
+#if defined(__MSDOS__) || defined(__EMX__) || defined(HAVE_DOS_PATHS)
       /* We need also to leave alone a trailing slash in "d:/".  */
       if (len > 3 || (len > 1 && v[1] != ':'))
 #endif
       if (len > 1 && p[-1] == '/')
 	--len;
 
+      /* Put the directory on the vpath list.  */
       if (len > 1 || *v != '.')
 	{
-	  v = savestring (v, len);
-
-	  /* Verify that the directory actually exists.  */
-
-	  if (dir_file_exists_p (v, ""))
-	    {
-	      /* It does.  Put it in the list.  */
-	      vpath[elem++] = dir_name (v);
-	      free (v);
-	      if (len > maxvpath)
-		maxvpath = len;
-	    }
-	  else
-	    /* The directory does not exist.  Omit from the list.  */
-	    free (v);
+          vpath[elem++] = dir_name (strcache_add_len (v, len));
+          if (len > maxvpath)
+            maxvpath = len;
 	}
 
       /* Skip over separators and blanks between entries.  */
@@ -285,40 +279,36 @@ construct_vpath_list (char *pattern, char *dirpath)
 	 entry, to where the nil-pointer terminator goes.
 	 Usually this is maxelem - 1.  If not, shrink down.  */
       if (elem < (maxelem - 1))
-	vpath = (char **) xrealloc ((char *) vpath,
-				    (elem + 1) * sizeof (char *));
+	vpath = xrealloc (vpath, (elem+1) * sizeof (const char *));
 
       /* Put the nil-pointer terminator on the end of the VPATH list.  */
-      vpath[elem] = 0;
+      vpath[elem] = NULL;
 
       /* Construct the vpath structure and put it into the linked list.  */
-      path = (struct vpath *) xmalloc (sizeof (struct vpath));
+      path = xmalloc (sizeof (struct vpath));
       path->searchpath = vpath;
       path->maxlen = maxvpath;
       path->next = vpaths;
       vpaths = path;
 
       /* Set up the members.  */
-      path->pattern = pattern;
-      path->percent = percent;
+      path->pattern = strcache_add (pattern);
       path->patlen = strlen (pattern);
+      path->percent = percent ? path->pattern + (percent - pattern) : 0;
     }
   else
-    {
-      /* There were no entries, so free whatever space we allocated.  */
-      free ((char *) vpath);
-      if (pattern != 0)
-	free (pattern);
-    }
+    /* There were no entries, so free whatever space we allocated.  */
+    /* MSVC erroneously warns without a cast here.  */
+    free ((void *)vpath);
 }
 
 /* Search the GPATH list for a pathname string that matches the one passed
    in.  If it is found, return 1.  Otherwise we return 0.  */
 
 int
-gpath_search (char *file, unsigned int len)
+gpath_search (const char *file, unsigned int len)
 {
-  char **gp;
+  const char **gp;
 
   if (gpaths && (len <= gpaths->maxlen))
     for (gp = gpaths->searchpath; *gp != NULL; ++gp)
@@ -328,57 +318,24 @@ gpath_search (char *file, unsigned int len)
   return 0;
 }
 
-/* Search the VPATH list whose pattern matches *FILE for a directory
-   where the name pointed to by FILE exists.  If it is found, we set *FILE to
-   the newly malloc'd name of the existing file, *MTIME_PTR (if MTIME_PTR is
-   not NULL) to its modtime (or zero if no stat call was done), and return 1.
-   Otherwise we return 0.  */
 
-int
-vpath_search (char **file, FILE_TIMESTAMP *mtime_ptr)
-{
-  register struct vpath *v;
+/* Search the given VPATH list for a directory where the name pointed to by
+   FILE exists.  If it is found, we return a cached name of the existing file
+   and set *MTIME_PTR (if MTIME_PTR is not NULL) to its modtime (or zero if no
+   stat call was done). Also set the matching directory index in PATH_INDEX
+   if it is not NULL. Otherwise we return NULL.  */
 
-  /* If there are no VPATH entries or FILENAME starts at the root,
-     there is nothing we can do.  */
-
-  if (**file == '/'
-#ifdef HAVE_DOS_PATHS
-      || **file == '\\'
-      || (*file)[1] == ':'
-#endif
-      || (vpaths == 0 && general_vpath == 0))
-    return 0;
-
-  for (v = vpaths; v != 0; v = v->next)
-    if (pattern_matches (v->pattern, v->percent, *file))
-      if (selective_vpath_search (v, file, mtime_ptr))
-	return 1;
-
-  if (general_vpath != 0
-      && selective_vpath_search (general_vpath, file, mtime_ptr))
-    return 1;
-
-  return 0;
-}
-
-
-/* Search the given VPATH list for a directory where the name pointed
-   to by FILE exists.  If it is found, we set *FILE to the newly malloc'd
-   name of the existing file, *MTIME_PTR (if MTIME_PTR is not NULL) to
-   its modtime (or zero if no stat call was done), and we return 1.
-   Otherwise we return 0.  */
-
-static int
-selective_vpath_search (struct vpath *path, char **file,
-                        FILE_TIMESTAMP *mtime_ptr)
+static const char *
+selective_vpath_search (struct vpath *path, const char *file,
+                        FILE_TIMESTAMP *mtime_ptr, unsigned int* path_index)
 {
   int not_target;
-  char *name, *n;
-  char *filename;
-  register char **vpath = path->searchpath;
+  char *name;
+  const char *n;
+  const char *filename;
+  const char **vpath = path->searchpath;
   unsigned int maxvpath = path->maxlen;
-  register unsigned int i;
+  unsigned int i;
   unsigned int flen, vlen, name_dplen;
   int exists = 0;
 
@@ -386,73 +343,73 @@ selective_vpath_search (struct vpath *path, char **file,
      If and only if it is NOT a target, we will accept prospective
      files that don't exist but are mentioned in a makefile.  */
   {
-    struct file *f = lookup_file (*file);
+    struct file *f = lookup_file (file);
     not_target = f == 0 || !f->is_target;
   }
 
-  flen = strlen (*file);
+  flen = strlen (file);
 
   /* Split *FILE into a directory prefix and a name-within-directory.
-     NAME_DPLEN gets the length of the prefix; FILENAME gets the
-     pointer to the name-within-directory and FLEN is its length.  */
+     NAME_DPLEN gets the length of the prefix; FILENAME gets the pointer to
+     the name-within-directory and FLEN is its length.  */
 
-  n = strrchr (*file, '/');
+  n = strrchr (file, '/');
 #ifdef HAVE_DOS_PATHS
   /* We need the rightmost slash or backslash.  */
   {
-    char *bslash = strrchr(*file, '\\');
+    const char *bslash = strrchr(file, '\\');
     if (!n || bslash > n)
       n = bslash;
   }
 #endif
-  name_dplen = n != 0 ? n - *file : 0;
-  filename = name_dplen > 0 ? n + 1 : *file;
+  name_dplen = n != 0 ? n - file : 0;
+  filename = name_dplen > 0 ? n + 1 : file;
   if (name_dplen > 0)
     flen -= name_dplen + 1;
 
-  /* Allocate enough space for the biggest VPATH entry,
-     a slash, the directory prefix that came with *FILE,
-     another slash (although this one may not always be
-     necessary), the filename, and a null terminator.  */
-  name = (char *) xmalloc (maxvpath + 1 + name_dplen + 1 + flen + 1);
+  /* Get enough space for the biggest VPATH entry, a slash, the directory
+     prefix that came with FILE, another slash (although this one may not
+     always be necessary), the filename, and a null terminator.  */
+  name = alloca (maxvpath + 1 + name_dplen + 1 + flen + 1);
 
   /* Try each VPATH entry.  */
   for (i = 0; vpath[i] != 0; ++i)
     {
       int exists_in_cache = 0;
+      char *p;
 
-      n = name;
+      p = name;
 
-      /* Put the next VPATH entry into NAME at N and increment N past it.  */
+      /* Put the next VPATH entry into NAME at P and increment P past it.  */
       vlen = strlen (vpath[i]);
-      bcopy (vpath[i], n, vlen);
-      n += vlen;
+      memcpy (p, vpath[i], vlen);
+      p += vlen;
 
       /* Add the directory prefix already in *FILE.  */
       if (name_dplen > 0)
 	{
 #ifndef VMS
-	  *n++ = '/';
+	  *p++ = '/';
 #endif
-	  bcopy (*file, n, name_dplen);
-	  n += name_dplen;
+	  memcpy (p, file, name_dplen);
+	  p += name_dplen;
 	}
 
 #ifdef HAVE_DOS_PATHS
       /* Cause the next if to treat backslash and slash alike.  */
-      if (n != name && n[-1] == '\\' )
-	n[-1] = '/';
+      if (p != name && p[-1] == '\\' )
+	p[-1] = '/';
 #endif
       /* Now add the name-within-directory at the end of NAME.  */
 #ifndef VMS
-      if (n != name && n[-1] != '/')
+      if (p != name && p[-1] != '/')
 	{
-	  *n = '/';
-	  bcopy (filename, n + 1, flen + 1);
+	  *p = '/';
+	  memcpy (p + 1, filename, flen + 1);
 	}
       else
 #endif
-	bcopy (filename, n, flen + 1);
+	memcpy (p, filename, flen + 1);
 
       /* Check if the file is mentioned in a makefile.  If *FILE is not
 	 a target, that is enough for us to decide this file exists.
@@ -499,7 +456,7 @@ selective_vpath_search (struct vpath *path, char **file,
 #else
 	  /* Clobber a null into the name at the last slash.
 	     Now NAME is the name of the directory to look in.  */
-	  *n = '\0';
+	  *p = '\0';
 
 	  /* We know the directory is in the hash table now because either
 	     construct_vpath_list or the code just above put it there.
@@ -520,7 +477,7 @@ selective_vpath_search (struct vpath *path, char **file,
 
 #ifndef VMS
 	  /* Put the slash back in NAME.  */
-	  *n = '/';
+	  *p = '/';
 #endif
 
 	  if (exists_in_cache)	/* Makefile-mentioned file need not exist.  */
@@ -543,31 +500,88 @@ selective_vpath_search (struct vpath *path, char **file,
             }
 
           /* We have found a file.
-             Store the name we found into *FILE for the caller.  */
-
-          *file = savestring (name, (n + 1 - name) + flen);
-
-          /* If we get here and mtime_ptr hasn't been set, record
+             If we get here and mtime_ptr hasn't been set, record
              UNKNOWN_MTIME to indicate this.  */
           if (mtime_ptr != 0)
             *mtime_ptr = UNKNOWN_MTIME;
 
-          free (name);
-          return 1;
+          /* Store the name we found and return it.  */
+
+          if (path_index)
+            *path_index = i;
+
+          return strcache_add_len (name, (p + 1 - name) + flen);
 	}
     }
 
-  free (name);
   return 0;
 }
+
+
+/* Search the VPATH list whose pattern matches FILE for a directory where FILE
+   exists.  If it is found, return the cached name of an existing file, and
+   set *MTIME_PTR (if MTIME_PTR is not NULL) to its modtime (or zero if no
+   stat call was done). Also set the matching directory index in VPATH_INDEX
+   and PATH_INDEX if they are not NULL.  Otherwise we return 0.  */
+
+const char *
+vpath_search (const char *file, FILE_TIMESTAMP *mtime_ptr,
+              unsigned int* vpath_index, unsigned int* path_index)
+{
+  struct vpath *v;
+
+  /* If there are no VPATH entries or FILENAME starts at the root,
+     there is nothing we can do.  */
+
+  if (file[0] == '/'
+#ifdef HAVE_DOS_PATHS
+      || file[0] == '\\' || file[1] == ':'
+#endif
+      || (vpaths == 0 && general_vpath == 0))
+    return 0;
+
+  if (vpath_index)
+    {
+      *vpath_index = 0;
+      *path_index = 0;
+    }
+
+  for (v = vpaths; v != 0; v = v->next)
+    {
+      if (pattern_matches (v->pattern, v->percent, file))
+        {
+          const char *p = selective_vpath_search (
+            v, file, mtime_ptr, path_index);
+          if (p)
+            return p;
+        }
+
+      if (vpath_index)
+        ++*vpath_index;
+    }
+
+
+  if (general_vpath != 0)
+    {
+      const char *p = selective_vpath_search (
+        general_vpath, file, mtime_ptr, path_index);
+      if (p)
+        return p;
+    }
+
+  return 0;
+}
+
+
+
 
 /* Print the data base of VPATH search paths.  */
 
 void
 print_vpath_data_base (void)
 {
-  register unsigned int nvpaths;
-  register struct vpath *v;
+  unsigned int nvpaths;
+  struct vpath *v;
 
   puts (_("\n# VPATH Search Paths\n"));
 
@@ -594,8 +608,8 @@ print_vpath_data_base (void)
     puts (_("\n# No general (`VPATH' variable) search path."));
   else
     {
-      register char **path = general_vpath->searchpath;
-      register unsigned int i;
+      const char **path = general_vpath->searchpath;
+      unsigned int i;
 
       fputs (_("\n# General (`VPATH' variable) search path:\n# "), stdout);
 
